@@ -128,12 +128,13 @@ import org.glassfish.grizzly.monitoring.jmx.JmxMonitoringAware;
 import org.glassfish.grizzly.monitoring.jmx.JmxObject;
 import org.glassfish.grizzly.utils.DelayedExecutor;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.exception.ExceptionUtils;
 import com.openexchange.http.grizzly.osgi.Services;
 import com.openexchange.http.grizzly.util.RequestTools;
 import com.openexchange.java.Charsets;
+import com.openexchange.marker.OXThreadMarker;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
-import com.openexchange.tools.exceptions.ExceptionUtils;
 
 /**
  * Filter implementation to provide high-level HTTP request/response processing.
@@ -263,14 +264,14 @@ public class OXHttpServerFilter extends HttpServerFilter implements JmxMonitorin
     private final ConcurrentMap<FilterChainContext, WatchInfo> pingMap;
     private final int pingDelay;
     private final int maxPingCount;
-    private volatile Ping ping;
+    private final Ping ping;
 
     // ------------------------------------------------------------ Constructors
 
     public OXHttpServerFilter(final ServerFilterConfiguration config, final DelayedExecutor delayedExecutor) {
         super(config, delayedExecutor);
         // Ping stuff
-        pingMap = new ConcurrentHashMap<FilterChainContext, WatchInfo>(512);
+        pingMap = new ConcurrentHashMap<FilterChainContext, WatchInfo>(512, 0.75f, 32);
         {
             final ConfigurationService service = Services.optService(ConfigurationService.class);
             pingDelay = null == service ? 90000 : service.getIntProperty("com.openexchange.http.grizzly.pingDelay", 90000);
@@ -283,15 +284,6 @@ public class OXHttpServerFilter extends HttpServerFilter implements JmxMonitorin
         reregisterForReadAttr = Grizzly.DEFAULT_ATTRIBUTE_BUILDER.createAttribute("HttpServerFilter.reregisterForReadAttr");
     }
 
-    /**
-     * Sets the ping
-     *
-     * @param ping The ping to set
-     */
-    public void setPing(final Ping ping) {
-        this.ping = ping;
-    }
-
     @Override
     @SuppressWarnings({"UnusedDeclaration"})
     public HttpHandler getHttpHandler() {
@@ -305,6 +297,19 @@ public class OXHttpServerFilter extends HttpServerFilter implements JmxMonitorin
 
     // ----------------------------------------------------- Methods from Filter
 
+    private static final OXThreadMarker DUMMY = new OXThreadMarker() {
+
+        @Override
+        public void setHttpRequestProcessing(boolean httpProcessing) {
+            // Nothing
+        }
+
+        @Override
+        public boolean isHttpRequestProcessing() {
+            return false;
+        }
+    };
+
     @SuppressWarnings({ "unchecked", "ReturnInsideFinallyBlock" })
     @Override
     public NextAction handleRead(final FilterChainContext ctx) throws IOException {
@@ -312,98 +317,108 @@ public class OXHttpServerFilter extends HttpServerFilter implements JmxMonitorin
         final Connection connection = ctx.getConnection();
 
         if (HttpPacket.isHttp(message)) {
+            OXThreadMarker threadMarker;
+            {
+                Thread t = Thread.currentThread();
+                threadMarker = t instanceof OXThreadMarker ? (OXThreadMarker) t : DUMMY;
+            }
+            threadMarker.setHttpRequestProcessing(true);
+            try {
+                // Otherwise cast message to a HttpContent
+                final HttpContent httpContent = (HttpContent) message;
 
-            // Otherwise cast message to a HttpContent
-            final HttpContent httpContent = (HttpContent) message;
+                Request handlerRequest = httpRequestInProcessAttr.get(connection);
 
-            Request handlerRequest = httpRequestInProcessAttr.get(connection);
+                if (handlerRequest == null) {
+                    // It's a new HTTP request
+                    final HttpRequestPacket request = (HttpRequestPacket) httpContent.getHttpHeader();
+                    final HttpResponsePacket response = request.getResponse();
+                    handlerRequest = OXRequest.create();
+                    handlerRequest.parameters.setLimit(getConfiguration().getMaxRequestParameters());
+                    httpRequestInProcessAttr.set(connection, handlerRequest);
+                    final Response handlerResponse = handlerRequest.getResponse();
 
-            if (handlerRequest == null) {
-                // It's a new HTTP request
-                final HttpRequestPacket request = (HttpRequestPacket) httpContent.getHttpHeader();
-                final HttpResponsePacket response = request.getResponse();
-                handlerRequest = OXRequest.create();
-                handlerRequest.parameters.setLimit(getConfiguration().getMaxRequestParameters());
-                httpRequestInProcessAttr.set(connection, handlerRequest);
-                final Response handlerResponse = handlerRequest.getResponse();
+                    handlerRequest.initialize(/* handlerResponse, */request, ctx, this);
+                    final SuspendStatus suspendStatus = handlerResponse.initialize(handlerRequest, response, ctx, suspendedResponseQueue, this);
 
-                handlerRequest.initialize(/* handlerResponse, */request, ctx, this);
-                final SuspendStatus suspendStatus = handlerResponse.initialize(handlerRequest, response, ctx, suspendedResponseQueue, this);
+                    HttpServerProbeNotifier.notifyRequestReceive(this, connection, handlerRequest);
 
-                HttpServerProbeNotifier.notifyRequestReceive(this, connection, handlerRequest);
+                    boolean wasSuspended = false;
+                    boolean pingInitiated = false;
 
-                boolean wasSuspended = false;
-                boolean pingInitiated = false;
+                    try {
+                        ctx.setMessage(handlerResponse);
 
-                try {
-                    ctx.setMessage(handlerResponse);
+                        if (!getConfiguration().isPassTraceRequest() && request.getMethod() == Method.TRACE) {
+                            onTraceRequest(handlerRequest, handlerResponse);
+                        } else {
+                            final HttpHandler httpHandlerLocal = httpHandler;
+                            if (httpHandlerLocal != null) {
+                                // Initiate ping (if required)
+                                if (ping != Ping.NONE && allowsPing(handlerRequest) && !isLongRunning(handlerRequest)) {
+                                    pingInitiated = initiatePing(handlerResponse, ctx, ping);
+                                }
 
-                    if (!getConfiguration().isPassTraceRequest() && request.getMethod() == Method.TRACE) {
-                        onTraceRequest(handlerRequest, handlerResponse);
-                    } else {
-                        final HttpHandler httpHandlerLocal = httpHandler;
-                        if (httpHandlerLocal != null) {
-                            // Initiate ping
-                            if (allowsPing(handlerRequest) && !isLongRunning(handlerRequest)) {
-                                pingInitiated = initiatePing(handlerResponse, ctx);
+                                // Handle HTTP message
+                                httpHandlerLocal.doHandle(handlerRequest, handlerResponse);
+                            }
+                        }
+                    } catch (Exception t) {
+                        handlerRequest.getRequest().getProcessingState().setError(true);
+
+                        if (!response.isCommitted()) {
+                            final ByteBuffer b = HtmlHelper.getExceptionErrorPage("Internal Server Error", "Grizzly/2.0", t);
+                            handlerResponse.reset();
+                            handlerResponse.setStatus(HttpStatus.INTERNAL_SERVER_ERROR_500);
+                            handlerResponse.setContentType("text/html");
+                            handlerResponse.setCharacterEncoding("UTF-8");
+                            final MemoryManager mm = ctx.getMemoryManager();
+                            final Buffer buf = Buffers.wrap(mm, b);
+                            handlerResponse.getOutputBuffer().writeBuffer(buf);
+                        }
+                    } catch (Throwable t) {
+                        ExceptionUtils.handleThrowable(t);
+                        LOGGER.log(Level.WARNING, "Unexpected error", t);
+                        throw new IllegalStateException(t);
+                    } finally {
+                        // don't forget to invalidate the suspendStatus
+                        wasSuspended = suspendStatus.getAndInvalidate();
+
+                        if (pingInitiated) {
+                            final WatchInfo watchInfo = pingMap.remove(ctx);
+                            if (null != watchInfo) {
+                                watchInfo.stopPing();
+                                watchInfo.timerTask.cancel(false);
+                                ctx.removeCompletionListener(watchInfo);
+                                connection.getMonitoringConfig().removeProbes(watchInfo);
+                                // Canceled timer task gets purged by CustomThreadPoolExecutorTimerService.PurgeRunnable
                             }
-                            // Handle HTTP message
-                            httpHandlerLocal.doHandle(handlerRequest, handlerResponse);
                         }
                     }
-                } catch (Exception t) {
-                    handlerRequest.getRequest().getProcessingState().setError(true);
 
-                    if (!response.isCommitted()) {
-                        final ByteBuffer b = HtmlHelper.getExceptionErrorPage("Internal Server Error", "Grizzly/2.0", t);
-                        handlerResponse.reset();
-                        handlerResponse.setStatus(HttpStatus.INTERNAL_SERVER_ERROR_500);
-                        handlerResponse.setContentType("text/html");
-                        handlerResponse.setCharacterEncoding("UTF-8");
-                        final MemoryManager mm = ctx.getMemoryManager();
-                        final Buffer buf = Buffers.wrap(mm, b);
-                        handlerResponse.getOutputBuffer().writeBuffer(buf);
+                    if (!wasSuspended) {
+                        return afterService(ctx, connection, handlerRequest, handlerResponse);
+                    } else {
+                        return ctx.getSuspendAction();
                     }
-                } catch (Throwable t) {
-                    ExceptionUtils.handleThrowable(t);
-                    LOGGER.log(Level.WARNING, "Unexpected error", t);
-                    throw new IllegalStateException(t);
-                } finally {
-                    // don't forget to invalidate the suspendStatus
-                    wasSuspended = suspendStatus.getAndInvalidate();
-
-                    if (pingInitiated) {
-                        final WatchInfo watchInfo = pingMap.remove(ctx);
-                        if (null != watchInfo) {
-                            watchInfo.stopPing();
-                            watchInfo.timerTask.cancel(false);
-                            ctx.removeCompletionListener(watchInfo);
-                            connection.getMonitoringConfig().removeProbes(watchInfo);
-                            // Canceled timer task gets purged by CustomThreadPoolExecutorTimerService.PurgeRunnable
-                        }
-                    }
-                }
-
-                if (!wasSuspended) {
-                    return afterService(ctx, connection, handlerRequest, handlerResponse);
                 } else {
-                    return ctx.getSuspendAction();
-                }
-            } else {
-                // We're working with suspended HTTP request
-                try {
-                    if (!handlerRequest.getInputBuffer().append(httpContent)) {
-                        // we don't want this thread/context to reset
-                        // OP_READ on Connection
+                    // We're working with suspended HTTP request
+                    try {
+                        if (!handlerRequest.getInputBuffer().append(httpContent)) {
+                            // we don't want this thread/context to reset
+                            // OP_READ on Connection
 
-                        // we have enough data? - terminate filter chain execution
-                        final NextAction action = ctx.getSuspendAction();
-                        ctx.completeAndRecycle();
-                        return action;
+                            // we have enough data? - terminate filter chain execution
+                            final NextAction action = ctx.getSuspendAction();
+                            ctx.completeAndRecycle();
+                            return action;
+                        }
+                    } finally {
+                        httpContent.recycle();
                     }
-                } finally {
-                    httpContent.recycle();
                 }
+            } finally {
+                threadMarker.setHttpRequestProcessing(false);
             }
         } else { // this code will be run, when we resume the context
             if (Boolean.TRUE.equals(reregisterForReadAttr.remove(ctx))) {
@@ -428,16 +443,16 @@ public class OXHttpServerFilter extends HttpServerFilter implements JmxMonitorin
     private static final String USM_USER_AGENT = "Open-Xchange USM HTTP Client";
     private static final String _NET_USER_AGENT = "Open-Xchange .NET HTTP Client";
     private static final String OXSTOR_USER_AGENT = "oxstor.dll";
+    private static final int MAX_UA_LEN = _NET_USER_AGENT.length();
 
     private static final Set<String> NO_PING = Collections.<String> unmodifiableSet(new HashSet<String>(Arrays.asList(USM_USER_AGENT, _NET_USER_AGENT, OXSTOR_USER_AGENT)));
 
     private boolean allowsPing(Request request) {
         String ua = request.getHeader("User-Agent");
-        return null != ua && !NO_PING.contains(ua);
+        return null != ua && (ua.length() > MAX_UA_LEN || !NO_PING.contains(ua));
     }
 
-    private boolean initiatePing(final Response handlerResponse, final FilterChainContext ctx) {
-        final Ping ping = this.ping;
+    private boolean initiatePing(final Response handlerResponse, final FilterChainContext ctx, final Ping ping) {
         if (ping != Ping.NONE) {
             final TimerService timerService = Services.optService(TimerService.class);
             if (null != timerService) {
